//JOHN PALMISANO		12-26-03
//LAST MODIFIED			3-12-04
//modified cereb.h (awesomified it) for mobot2004
//WORKS ON NEW CEREBELLUM

#include <16F877.h>
#include <877reg.h>
#include "adc.h"
#include <pwm298jsp2_new.h>//for new cerebellum

#fuses HS, NOWDT
#use rs232(baud=115200, parity=N, bits=8, xmit=PIN_C6, rcv=PIN_C7)

// for LEDs
#define YELLOW 1
#define GREEN 0

// for buttons
#define A 0
#define B 1

// for wheels
#define LEFT 0
#define RIGHT 1

// ADC ports
#define A0 0
#define A1 1
#define A2 2
#define A3 3
#define A5 4
#define E0 5
#define E1 6
#define E2 7

// SERVO control vars/functions - not for the faint of heart!
// Specify Servo Port
#define SERVOPORT 3
#define SERVO_PORT PORTD
#define SERVO_TRIS TRISD

char servo_state = 0;
char servo_count = 0;
char servo_curr = 0;
//char servo_mask = 0;
char servo_switch = 1;
char servo_pos[8] = {0,0,0,0,0,0,0,0}; // servo_pos[7] is reserved for a boolean check if heat is found
char servo_free = 0;
//#bit servo_free = _servo_free.0

#INT_RTCC
// servo control variables
void servo_isr(void) {

	servo_free = 0;

	if (servo_state == 1) {
		// next intr in 0.6 ms
		TMR0 = 180;

		if (servo_pos[servo_curr] != 0)
			SERVO_PORT = SERVO_PORT | servo_switch; // output hi
		servo_switch = servo_switch << 1;

		if (servo_switch == 0) servo_switch = 1;

		servo_state = servo_state + 1;
		servo_count = 0;
		servo_free = 1;
	}

	else if (servo_state == 2) {
		TMR0 = 255 - servo_pos[servo_curr];
		servo_state = 3;
	}

	else if (servo_state == 3) {
		SERVO_PORT = 0; // output lo
		TMR0 = servo_pos[servo_curr];
		servo_curr = servo_curr + 1;
		servo_curr = servo_curr & 0x07;
		servo_state = 1;
	}
	T0IF = 0;
}

inline void servo_init(void) {
    OPTION_REG = 132; // bit 7 = 1 for portB pull-up and bit2 is for prescaler
	// TMR0 @ 1:32, prescaler to timer0 (for servo control)
    TRISD = 0; // port D as outputs	for servo commands!
    T0IE = 1; //T0IE = 1 ie enable TMR0 overflow bit - T0IE is bit 5
    	 
	servo_state = 1;
	GIE = 1;
	
    delay_ms(1);
}

// turn a specified(0 or 1) LED on.
void LEDon(int which)
{
  /*if(which == 0)
    output_high(PIN_B0);

  else
    output_high(PIN_B2);*/
  if(which == 1)
    output_high(PIN_B2);
}

// turn a specified(0 or 1) LED off.
void LEDoff(int which)
{
  /*if(which == 0)
    output_low(PIN_B0);

  else
    output_low(PIN_B2);*/
  if(which == 1)
    output_low(PIN_B2);
}

// takes a -90 to 90 angle, converts to servo 255 to 0
void scan(signed int angle)
{
  int pulseWS;
  //int pulseSS;
  
  // converts -100 to 100 to the full range of servo motion
  // centers at 127, the 0 spot of a servo.
  pulseWS = 127 + (int)((4*((signed long int)angle))/3);
  //pulseSS = 127 + (int)((4*((signed long int)sensorsteer))/3);

  // sets the given servo to the given position.
  servo_pos[0]=pulseWS-8;//modified because servo misaligned
  //servo_pos[1]=pulseSS;
}

// read from a given analog port
unsigned int analog(int which)
{
	unsigned int sense;

	sense = adc_read(which);
	delay_ms(1);

	return sense;
}

inline int button(int which)
{
   int state;
   if(which == A)
	state = input(PIN_B4);
   //else//disabled to use B5 for interrupts
	//state = input(PIN_B5);

   return state;
}

inline void cereb_init()
{
  GIE = 1; // Enable Interrupts (servos only work with enabled interrupts)

  servo_init();  // Initialize servos

  servo_state = 1;  // Enable servos
  
  pwm_setup();
  
  adc_init(0);

  //LEDoff(GREEN);
  LEDoff(YELLOW);

  delay_ms(1);
}